// C++ to Add Arc Text
//   lines starting with a // are comments
#include "stdafx.h"
#include "VCADDTools.h"
#include "ArcText.h"
#include <math.h>

//  file scope variables
static short iError;
static Point2D ptCenter;
static Point2D ptRadius;
static double dRadius;
static double dAngle;
static long hEntRubber;

void WINAPI ArcText(char* szDllCommandLine)
{
    hEntRubber = -1;    //  initialize

    //  required parameter but unused by this function
    //  circumvent the unreferenced formal parameter warning
    UNUSED_PARAMETER(szDllCommandLine);

    //  ArcText tool will draw text curved along an arc entered 
    //  by the user.  The tool has 3 states:
    //      (1) place center point of the arc
    //      (2) place radius and text start point
    //      (3) type text
    //  ArcText supports the BackSpace key and undo/redo.
    //  ArcText is placed as a group of individual characters.

    //  Optional - save the settings to be restored later
    VCSaveSettings();

    //  Increment the undo level to support undo/redo.
    //  Optional - use VCBeginOperation() instead.  
    //      VCBeginOperation() does only two things - it 
    //      increments the undo level and saves the current end 
    //      of the database for cleanup when done.  The ArcText 
    //      tool uses a rubber-band which is appended at the end
    //      of the database, so we can use that entity handle for 
    //      cleanup instead.
    VCIncrementUndoLevel(&iError);

    //  Start a user tool and define the status bar command line
    //  prompts for the ArcText tool.  The states are 0, 1, 2.
    //  The first prompt for state 0 is set by VCSetUserTool()
    //  and the next two prompts are set by VCSetPrompt().
    VCSetUserTool(2, "ArcText", "Enter Center Point");
    VCSetPrompt(1, "Enter Start Point of Text");
    VCSetPrompt(2, "Enter Text, press [Back] key to re-type, [Enter] key to finish, [Esc] key to abort");

    //  Set alert events so our tool gets notifications from 
    //  Visual CADD about user interface events, other tool
    //  operations, and Windows messages.  We are requesting 
    //  alerts on mouse-down (point placement), mouse-move (for
    //  rubber-bands), tool pen-up (when tool ends normally),
    //  tool abort (when user aborts or another tool starts), and
    //  Windows messages.  Each of these has a corresponding 
    //  callback function below.
    VCSetAlertAppDllEx(&iError, "API_Help_C.dll", "ArcText", 
        ALERT_APP_UTOOL_MOUSEDOWN + 
        ALERT_APP_UTOOL_MOUSEMOVE + 
        ALERT_APP_UTOOL_PENUP +
        ALERT_APP_UTOOL_ABORT,
        ALERT_APP_EX_PRE_TRANSLATE_GET_MESSAGE);
}

void WINAPI ArcText_MouseDown(WPARAM wParam, LPARAM lParam)
{
    //  required parameter but unused by this function
    //  circumvent the unreferenced formal parameter warning
    UNUSED_PARAMETER(wParam);
    UNUSED_PARAMETER(lParam);

    //  get the tool state so we know the state of the tool
    short iState;
    iState = VCGetUserToolState(&iError);

    //  Visual CADD increases the tool state of user tools after 
    //  mouse clicks.  Tool state 1 means the tool state was 
    //  previously 0 and the user has clicked for the center of
    //  the arc.     
    if (iState == 1)
    {
        //  get the mouse-click point from Visual CADD
        VCGetUserToolLBDown(&iError, &ptCenter);

        //  add circle and line (radius) entities to be the 
        //  rubber-band.  Save the rubber-band entity handle for
        //  later use.
        VCAddCircleEntity(&iError, -1, ptCenter, ptCenter);
        VCLastEntity(&iError, &hEntRubber);
        VCAddLineEntity(&iError, -1, ptCenter, ptCenter);
    }

    //  Tool state 2 means the tool state was previously 1 and 
    //  the user has clicked for the start of the text.
    else if (iState == 2)
    {
        //  get the mouse-click point from Visual CADD
        VCGetUserToolLBDown(&iError, &ptRadius);

        //  calculate the radius distance and the angle.
        double dX;
        double dY;

        dX = ptRadius.x - ptCenter.x;
        dY = ptRadius.y - ptCenter.y;

        dRadius = sqrt(dX * dX + dY * dY);
        dAngle = atan2(dY, dX);
    }

    //  Tool state 3 means the tool state was previously 2 and 
    //  the user has clicked to finish the tool.
    else if (iState == 3)
    {
        VCPenUp();
    }
}

void WINAPI ArcText_MouseMove(WPARAM wParam, LPARAM lParam)
{
    //  required parameter but unused by this function
    //  circumvent the unreferenced formal parameter warning
    UNUSED_PARAMETER(wParam);
    UNUSED_PARAMETER(lParam);

    //  get the tool state so we know the state of the tool
    short iState;
    iState = VCGetUserToolState(&iError);

    //  use the mouse move to draw the rubber-band while the user
    //  is choosing the arc start point.  Tool state 1 means the 
    //  user has already clicked the center so we will draw the
    //  rubber-band to the radius at the current mouse location.
    if (iState == 1)
    {
        //  drawing rubber-bands involves erasing the old 
        //  rubber-band and drawing the new one.  The function
        //  VCDrawCurrentEntityXOR() is used to draw entities 
        //  using the user's rubber-band color.  The XOR means 
        //  if it is already drawn on-screen XOR will remove it; 
        //  if it is not already drawn on-screen XOR will draw it.

        //  erase the circle and line (radius) rubber-bands
        VCSetCurrentEntity(&iError, hEntRubber);
        VCDrawCurrentEntityXOR(&iError);

        VCSetCurrentEntity(&iError, hEntRubber + 1);
        VCDrawCurrentEntityXOR(&iError);

        //  get the mouse location point from Visual CADD
        VCGetUserToolMouseMove(&iError, &ptRadius);

        //  set the new point in the circle and line (radius) 
        //  rubber-band entities and draw them
        VCSetCurrentEntity(&iError, hEntRubber);
        VCSetCurrentEntityPoint(&iError, 1, &ptRadius);
        VCDrawCurrentEntityXOR(&iError);

        VCSetCurrentEntity(&iError, hEntRubber + 1);
        VCSetCurrentEntityPoint(&iError, 1, &ptRadius);
        VCDrawCurrentEntityXOR(&iError);
    }
}

void WINAPI ArcText_Abort(WPARAM wParam, LPARAM lParam)
{
    //  required parameter but unused by this function
    //  circumvent the unreferenced formal parameter warning
    UNUSED_PARAMETER(wParam);
    UNUSED_PARAMETER(lParam);

    //  ArcText tool has aborted, so we need to cleanup the 
    //  database.

    //  Call the CleanUp function to handle routine 
    //  processing
    CleanUp();

    //  If ArcText made it as far as a rubber-band, then erase
    //  from the screen any text.
    if (hEntRubber != -1)
    {
        //  Get the last entity handle
        long hEntLast;
        VCLastEntity(&iError, &hEntLast);

        //  Loop through the text entities
        for (long hEnt = hEntRubber + 2; hEnt <= hEntLast; hEnt++)
        {
            //  Erase each text entity and draw it on-screen
            //  to erase it from screen
            VCSetCurrentEntity(&iError, hEnt);
            VCSetCurrentErased(&iError);
            VCDrawCurrentEntity(&iError);
        }

        //  Truncate the database starting with the rubber-band
        //  to restore the database to its original condition.
        VCTruncFrom(&iError, hEntRubber);
    }

    //  Decrement the undo level since the ArcText was aborted 
    //  and no entities were added.  Optional - if you used 
    //  VCBeginOperation(), then you should call 
    //  VCAbortOperation() which does the same VCTruncFrom() and 
    //  VCDecrementUndoLevel().
    VCDecrementUndoLevel(&iError);
}

BOOL WINAPI ArcText_PreTranslateGetMessage(MSG* pMsg)
{
    //  PreTranslateGetMessage processes posted messages, usually
    //  keyboard messages, but any posted messages may be 
    //  processed.

    //  The function return will tell Visual CADD if we have 
    //  processed the message and whether it should process it.
    //  The default is FALSE, we have not processed it.  Setting 
    //  to TRUE tells Visual CADD to not process the message.

    //  get the tool state so we know the state of the tool
    short iState;
    iState = VCGetUserToolState(&iError);

    //  VK_ESCAPE is Esc key.  Abort the ArcText tool 
    //  anytime the user presses Esc.
    if ((pMsg->message == WM_KEYDOWN) && (pMsg->wParam == VK_ESCAPE))
    {
        VCAbort();
        return TRUE;
    }

    //  Once the tool reaches state 2, we will process keyboard
    //  input as the user types.
    if (iState == 2)
    {
        if (pMsg->message == WM_KEYDOWN)
        {
            if (pMsg->wParam == VK_RETURN)
            {
                //  VK_RETURN is Enter key.
                //  Our Finish function will complete the ArcText.
                VCPenUp();
                return TRUE;
            }
            else if (pMsg->wParam == VK_BACK)
            {
                //  VK_BACK is backspace key.
                //  Our BackSpace function will remove the last
                //  typed character.
                BackSpace();
                return TRUE;
            }
        }
        else if (pMsg->message == WM_CHAR)
        {
            if (pMsg->wParam >= 32)
            {
                //  characters 32 and above are printable 
                //  so we add them to our ArcText by calling our 
                //  TextIt function.
                char strChar[2] = {0, 0};
                strChar[0] = char(pMsg->wParam);
                TextIt(strChar);
            }
            return TRUE;
        }
    }

    //  We have not processed the message, let Visual CADD.
    return FALSE;
}

void TextIt(char* strText)
{
    //  TextIt is called by the PreTranslateGetMessage after we 
    //  are in state 2, meaning we are accepting text input.

    //  create point objects and variables for calculation 
    Point2D pt0;

    char strChar[2] = {0, 0};
    long hEntLast;
    long iLen;
    long i;
    double dSpan;
    double dWidth;

    //  We are using the TextIt function to process one character
    //  at a time.  However, the function can also process full 
    //  length text strings.  We get the length of the string, 
    //  1 in the case of the current ArcText tool.
    iLen = strlen(strText);

    //  Loop through the string one character at a time
    for (i = 0; i < iLen; i++)
    {
        //  get a single character and set Visual CADD text to 
        //  use that character
        strChar[0] = strText[i];
        VCSetTextString(&iError, strChar);

        //  Total character width is the width of the text entity
        //  using that character plus character spacing
        dWidth = GetStringWidth();

        //  Calculate the text placement point from the center,
        //  radius, and angle
        pt0.x = ptCenter.x + dRadius * cos(dAngle);
        pt0.y = ptCenter.y + dRadius * sin(dAngle);

        //  Calculate how much angle is spanned by the character
        //  width
        dSpan = atan2(dWidth, dRadius);

        //  The standard angle in Visual CADD is measured 
        //  counter-clockwise from 3 o'clock whereas text 
        //  rotation angle of 0.0 means upright text, or 12 
        //  o'clock, a difference of 90 degrees.  The API uses 
        //  radians for angles, so 90 degrees is pi / 2 and we 
        //  adjust the text rotation angle by that amount.
        VCSetTextRot(&iError, dAngle - dSpan / 2.0 - HALFPI);

        //  spacing requires left-justified text
        VCSetTextJustify(&iError, 'L');

        //  Add the text at the calculated point
        VCAddTextEntity(&iError, -1, pt0);

        //  Adjust the angle clockwise for the next character
        dAngle = dAngle - dSpan;

        //  Draw the newly placed character
        VCLastEntity(&iError, &hEntLast);
        VCSetCurrentEntity(&iError, hEntLast);
        VCDrawCurrentEntity(&iError);
    }
}

void ArcText_PenUp(void)
{
    //  ArcText tool has completed normally, so we need to 
    //  cleanup the database and do final processing.

    //  Get the last entity handle
    long hEntLast;
    VCLastEntity(&iError, &hEntLast);

    //  Check that the user has entered some text, not just
    //  pressed Enter.
    if ((hEntRubber == -1) || (hEntLast < hEntRubber + 2))
    {
        //  There was no text entered, so Abort instead.
        VCAbort();
    }
    else
    {
        //  Call the CleanUp function to handle routine 
        //  processing
        CleanUp();

        //  clear selection
        VCClearSelection();

        //  Loop through the text entities and select them
        for (long hEnt = hEntRubber + 2; hEnt <= hEntLast; hEnt++)
        {
            VCSetCurrentEntity(&iError, hEnt);
            VCSetCurrentSelected(&iError);
        }

        //  Purge the database to cleanup redo conflicts possibly
        //  left by previous tools.  Optional - if you used 
        //  VCBeginOperation(), then you should call 
        //  VCEndOperation() which does the same VCPurgeRedo().
        VCPurgeRedo(&iError);

        //  Group the ArcText entities so they stay together
        //  when selected
        VCGroupSelected();
    }
}

double GetCharacterSpace(void)
{
    double dCharSp;
    BYTE bType;

    //  Get the character space setting so we can calculate
    //  how to space the text around the arc.  
    VCGetTextFontNameAndType(&iError, NULL, &bType);
    if (bType == 1)
    {
        //  Font type 1 is a vector font.  The character spacing
        //  is a percentage of the text height.
        double dHeight;
        double dCharSpPerCent;
        dHeight = VCGetTextHeight(&iError);
        dCharSpPerCent = VCGetTextCharSpace(&iError);
        dCharSp = dHeight * dCharSpPerCent;
    }
    else
    {
        //  Font type 0 is a TrueType font.  The character 
        //  spacing is built directly into TrueType character 
        //  definitions so there is no additional spacing.
        dCharSp = 0.0;
    }

    return dCharSp;
}

double GetStringWidth(void)
{
    //  create point objects and variables for calculation 
    Point2D pt0;
    Point2D pt1;

    //  use rotation angle 0.0 to place a temporary text 
    //  entity we can use to find the width of the character.
    VCSetTextRot(&iError, 0.0);
    VCAddTextEntity(&iError, -1, ptCenter);

    //  VCEntityExtents() gets the size of an entity.  After
    //  we get the size, we can use VCTruncFrom() to delete
    //  this temporary entity.
    long hEntLast;
    VCLastEntity(&iError, &hEntLast);
    VCEntityExtents(&iError, hEntLast, &pt0, &pt1);
    VCTruncFrom(&iError, hEntLast);

    //  Total character width is the width of the text entity
    //  using that character plus character spacing
    return (pt1.x - pt0.x + GetCharacterSpace());
}

void BackSpace(void)
{
    //  BackSpace is called by the PreTranslateGetMessage after
    //  we are in state 2, meaning we are accepting text input,
    //  and the user has pressed the Back key.

    //  create variables for calculation 
    long hEntLast;
    double dSpan;
    double dWidth;

    //  Get the last entity handle
    VCLastEntity(&iError, &hEntLast);

    //  Before processing a backspace key, be sure our text 
    //  string is not already empty.  The rubber-band is two
    //  entities, so the last entity must be after the second
    //  rubber-band entity.
    if (hEntLast > hEntRubber + 1)
    {
        //  Match the last entity so we can use its text string, 
        //  actually an individual character
        VCSetCurrentEntity(&iError, hEntLast);
        VCMatchCurrentEntity(&iError);

        //  Total character width is the width of the text entity
        //  using that character plus character spacing
        dWidth = GetStringWidth();

        //  Calculate how much angle is spanned by the character
        //  width and adjust the angle counter-clockwise back 
        //  to the previous character.
        dSpan = atan2(dWidth, dRadius);
        dAngle = dAngle + dSpan;

        //  Erase the last entity, draw it so it erases from the
        //  screen, and truncate the database to permanently 
        //  remove it.
        VCLastEntity(&iError, &hEntLast);
        VCSetCurrentEntity(&iError, hEntLast);
        VCSetCurrentErased(&iError);
        VCDrawCurrentEntity(&iError);
        VCTruncFrom(&iError, hEntLast);
    }
}

void CleanUp(void)
{
    //  If ArcText made it as far as a rubber-band, then erase
    //  from the screen the two rubber-band entities.  CleanUp is
    //  called by both ToolComplete and ToolAbort, so we do not
    //  know whether the database will be purged or truncated.
    //  Thus we use VCSetCurrentEntityPurged() to both erase the
    //  entities and mark them for purging.  Draw the erased
    //  entities to remove them from the screen.
    if (hEntRubber != -1)
    {
        VCSetCurrentEntity(&iError, hEntRubber);
        VCSetCurrentEntityPurged(&iError, 1);
        VCDrawCurrentEntity(&iError);

        VCSetCurrentEntity(&iError, hEntRubber + 1);
        VCSetCurrentEntityPurged(&iError, 1);
        VCDrawCurrentEntity(&iError);
    }

    //  We are done so clear the alert events so Visual CADD
    //  no longer sends the alerts.
    VCClearAlertAppDll(&iError, "API_Help_C.dll", "ArcText");

    //  Restore the saved settings so the user settings are as 
    //  they began.
    VCRestoreSettings();
}
