/*SDOC*************************************************************

  Module:	VCADDHelper.cpp

  Copyright 2005 TriTools Partners L.L.C.

  Current Version: 5.0.1
  
  Revision Date: 02/01/05

  Description:	Helper routines for Visual CADD C/C++ API programing
  
  Notes:	All the functions in VCADDHelper.cpp are compatible
					with Visual CADD version 2.0.1.0 and above.
	
	See VCADDHelper.h for Revision Record:

*************************************************************EDOC*/
#include "stdafx.h"
#include <stdlib.h>
#include "VCADDHelper.h"

// Create a variable that is shared between instances of this DLL
#pragma data_seg("DataSeg_Shared") 
// This data is shared between processes 
int Shared_iInitCounter = 0;
#pragma data_seg() // turn off sharing 

// Instruct the linker to make the Shared section 
// readable, writable and shared 
#pragma comment(linker, "/section:DataSeg_Shared,rws")

// globals 
HMODULE hInstVCBASE32;
HMODULE hInstVCDB32;
HMODULE hInstVCMAIN32;
HMODULE hInstVCTOOL32;
HMODULE hInstVCDLG32;
HMODULE hInstVCTRAN32;


// LoadVCADDDLLS(...) will only return TRUE if this the first DLL_PROCESS_ATTACH event
// and the Visual CADD dlls being loaded are at least the minimum version specified.
BOOL LoadVCADDDLLS(short* iError, BYTE bMajorVersion, BYTE bMinorVersion, BYTE bMinorDotVersion, BYTE bMinorInternalVersion)
{
	// set iError with the init counter, if this is the first
	// DLL_PROCESS_ATTACH event then the value will  be 0
	*iError = (short)Shared_iInitCounter;
	
	// only load the dlls if this is the first DLL_PROCESS_ATTACH event
	if(Shared_iInitCounter++ == 0)
	{
		// if any of the Visual CADD dlls fail set the iError to negative value
		// based on why dll load failed and return FALSE
		*iError = -1;

		// starting with Windows 2000 we need to make sure we are loading the correct
		// dlls we need to specify the full path
		char szPath[_MAX_PATH + 1];

		// calling GetModuleFileName(...) with hModule set to NULL retrieves
		// the path to the executable loading this dll
		if(GetModuleFileName(NULL, szPath, _MAX_PATH))
		{
			*iError = 0;
			
			// szPath will contain the full path and file name, we
			// need replace the file name with the dll name
			char szDrive[_MAX_DRIVE + 1];
			char szDir[_MAX_DIR + 1];
			_splitpath_s(szPath, szDrive, sizeof(szDrive), szDir, sizeof(szDir), NULL, NULL, NULL, NULL);


			// append the dll name to the path
			_makepath_s(szPath, sizeof(szPath), szDrive, szDir, "vcbase32", ".dll");
			hInstVCBASE32 = ::LoadLibrary(szPath);
			if (hInstVCBASE32 == NULL)
			{
				*iError = -2;
			}

			_makepath_s(szPath, sizeof(szPath), szDrive, szDir, "vcdb32", ".dll");
			hInstVCDB32 = ::LoadLibrary(szPath);
			if (hInstVCDB32 == NULL)
			{
				*iError = -3;
			}
			
			_makepath_s(szPath, sizeof(szPath), szDrive, szDir, "vcmain32", ".dll");
			hInstVCMAIN32 = ::LoadLibrary(szPath);
			if (hInstVCMAIN32 == NULL)
			{
				*iError = -4;
			}
			
			_makepath_s(szPath, sizeof(szPath), szDrive, szDir, "vctool32", ".dll");
			hInstVCTOOL32 = ::LoadLibrary(szPath);
			if (hInstVCTOOL32 == NULL)
			{
				*iError = -5;
			}
			_makepath_s(szPath, sizeof(szPath), szDrive, szDir, "vcdlg32", ".dll");					
			hInstVCDLG32 = ::LoadLibrary(szPath);
			if (hInstVCDLG32 == NULL)
			{
				*iError = -6;
			}
			_makepath_s(szPath, sizeof(szPath), szDrive, szDir, "vctran32", ".dll");
			hInstVCTRAN32 = ::LoadLibrary(szPath);
			if (hInstVCTRAN32 == NULL)
			{
				*iError = -7;
			}
			
			if(*iError == 0)
			{
				if(!MinimumVCADDVersionCheck( bMajorVersion, bMinorVersion, bMinorDotVersion, bMinorInternalVersion))
				{
					// failed the minimum version test
					*iError = -8;
				}
			}
			if(*iError != 0)
			{
				// need to preserve the exsisting iError code
				short iFreeError;
				FreeVCADDDLLS(&iFreeError);
			}
		}
	}
	
	// return TRUE only if this is the first DLL_PROCESS_ATTACH event
	// and there is no iError value
	return (*iError == 0 && Shared_iInitCounter == 1);
}

// FreeVCADDDLLS(...) will only return TRUE if this the final DLL_PROCESS_DETACH event
BOOL FreeVCADDDLLS(short* iError)
{
	// check the shared data's init counter to see if this is
	// the last instance of the dll
	// only release the dlls if this is the last instance
	if(Shared_iInitCounter-- == 1)
	{
		// free the Visual CADD library modules
		if (hInstVCTRAN32)
		{
			::FreeLibrary(hInstVCTRAN32);
			hInstVCTRAN32 = NULL;
		}
		if (hInstVCDLG32)
		{
			::FreeLibrary(hInstVCDLG32);
			hInstVCDLG32 = NULL;
		}
		if (hInstVCTOOL32)
		{
			::FreeLibrary(hInstVCTOOL32);
			hInstVCTOOL32 = NULL;
		}
		if (hInstVCMAIN32)
		{
			::FreeLibrary(hInstVCMAIN32);
			hInstVCMAIN32 = NULL;
		}
		if (hInstVCDB32)
		{
			::FreeLibrary(hInstVCDB32);
			hInstVCDB32 = NULL;
		}
		if (hInstVCBASE32)
		{
			::FreeLibrary(hInstVCBASE32);
			hInstVCMAIN32 = NULL;
		}
	}
	
	// set iError with the init counter, if the dll is being
	// freed then this will be 0
	*iError = (short)Shared_iInitCounter;
	
	// return TRUE only if the dll was freed
	return Shared_iInitCounter == 0;
}

// returns the handle for the dll
HMODULE GethInstVCMAIN32(void)
{
	return hInstVCMAIN32;
}

// returns the handle for the dll
HMODULE GethInstVCTOOL32(void)
{
	return hInstVCTOOL32;
}

// returns the handle for the dll
HMODULE GethInstVCDLG32(void)
{
	return hInstVCDLG32;
}

// returns the handle for the dll
HMODULE GethInstVCTRAN32(void)
{
	return hInstVCTRAN32;
}


// MinimumVCADDVersionCheck(...) will return TRUE the Visual CADD dlls
// loaded are at least the minimum version specified.
BOOL MinimumVCADDVersionCheck(BYTE bMajorVersion, BYTE bMinorVersion, BYTE bMinorDotVersion, BYTE bMinorInternalVersion)
{
	DWORD dwVCADDVersion = 0;
		
	// if the Visual CADD version is less then 5.0.0.0 we have to do this the hard way
	if(VCGetMajorVersion() < 5)
	{
		dwVCADDVersion = (DWORD)MAKELONG(MAKEWORD((BYTE)VCGetMinorInternalVersion(), (BYTE)VCGetMinorDotVersion()), MAKEWORD((BYTE)VCGetMinorVersion(), (BYTE)VCGetMajorVersion()));
	}
	else
	{
		dwVCADDVersion = VCGetFullVersion();
	}	
	return dwVCADDVersion >= (DWORD)MAKELONG(MAKEWORD(bMinorInternalVersion, bMinorDotVersion), MAKEWORD(bMinorVersion, bMajorVersion));
}

